/*
 * Author: Celforyon (Alexis Pereda)
 */

#ifndef __CFYZSHPROMPT_UTF8_H__
#define __CFYZSHPROMPT_UTF8_H__

/**
 * Data types and constants
 **/

/* Currently, UTF-8 is defined to use at most 4 bytes */
#define UTF8_MAX_BYTES					4

/* An UTF-8 char / string
 * To reduce the time cost of UTF-8, there is no dynamic allocation
 * so each character takes maximum memory size.
 */
typedef struct { char u[UTF8_MAX_BYTES];	}	utf8_char;
typedef utf8_char*								utf8_string;

/**
 * Displaying
 **/

/* Displays an UTF-8 char */
void utf8_putc(const utf8_char c);
/* Displays an UTF-8 string */
void utf8_puts(utf8_string str);

/* Displays an UTF-8 char for ZSH (surround by %1{ } if not ASCII) */
void utf8_zputc(const utf8_char c);
/* Displays an UTF-8 string for ZSH (call utf8_zputc) */
void utf8_zputs(utf8_string str);

/**
 * String manipulation
 **/

/* Sets an UTF-8 char from an UTF-8-ASCIIZ string.
 * This string must starts with a valid UTF-8 character
 * This function will return the number of char read to make the UTF-8 char
 * and 0 if it was a NULL character
 */
int utf8_setc(utf8_char *c, const char *s);

/* Sets an UTF-8 string from an UTF-8-ASCIIZ string.
 * This string must starts with a valid UTF-8 character
 * and contains only valid UTF-8 characters in sequence
 * This function will return the number of char read from s
 */
int utf8_set(utf8_string us, const char *s);

/* Acts like utf8_set but stopping at nth UTF-8 char */
int utf8_nset(utf8_string us, const char *s, int n);

/* Concats an UTF-8-ASCIIZ string to an UTF-8 string
 * See utf8_set for the requirements
 */
int utf8_cat(utf8_string us, const char *s);

/* Acts like utf8_cat but stopping at nth char */
int utf8_ncat(utf8_string us, const char *s, int n);

/* Copies from in to */
void utf8_setcu(utf8_char *to, const utf8_char *from);

/* Copies from in to
 * Warning: to must be at least sized to from
 */
void utf8_setu(utf8_string to, utf8_string from);

/* Acts like utf8_set8 but stopping at nth char */
void utf8_nsetu(utf8_string to, utf8_string from, int n);

/* Copies from at the end of to
 * Warning: to must have at least the size of from after last not null char
 */
void utf8_catu(utf8_string to, const utf8_string from);

/* Acts like utf8_cat8 but stopping at nth char */
void utf8_ncatu(utf8_string to, const utf8_string from, int n);

/* Splits us by sep and return substring after each call */
utf8_string utf8_sep(utf8_string us, utf8_char sep, int *s);

/**
 * String information
 **/

/* Returns the number of utf8_char before the null one */
int utf8_strlen(utf8_string us);

/* Returns 0 if a and b are equal */
int utf8_ccmp(utf8_char a, utf8_char b);

/* Returns 0 if a and b are equal */
int utf8_cmp(utf8_string a, utf8_string b);

/* Returns 0 if n first chars are equal */
int utf8_ncmp(utf8_string a, utf8_string b, int n);

/* Returns 1 if us starts with s */
int utf8_startswith(utf8_string us, utf8_string s);

#endif
